#
# Copyright (C) 2008 Francesco Salvestrini
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import os
import sys

from   Debug      import *
from   Trace      import *
from   Command    import *
import Exceptions

from   ID         import *

def description() :
    return "edit a node"

def do(configuration, arguments) :
    command = Command("edit")
    command.add_option("-i", "--id",
                       action = "store",
                       type   = "string",
                       dest   = "id",
                       help   = "specify node id to edit")
    command.add_option("-e", "--editor",
                       action = "store",
                       type   = "string",
                       dest   = "editor",
                       help   = "specify editor to use")

    (opts, args) = command.parse_args(arguments)

    # Parameters setup
    if (opts.id == None) :
        raise Exceptions.MissingParameters("node id")

    editor = None
    # Prefer parameter
    if (editor == None) :
        editor = opts.editor
    # Fall-back to configuration
    if (editor == None) :
        editor = configuration.get(command.name, 'editor', raw = True)
    # Fall-back to the environment
    if (editor == None) :
        editor = os.environ["EDITOR"]
    # Finally bang with error
    if (editor == None) :
        raise MissingParameters("editor")

    # Work
    id = ID(opts.id)

    debug("Success")

# Test
if (__name__ == '__main__') :
    debug("Test completed")
    sys.exit(0)
